import 'package:flutter/widgets.dart';
import 'package:get_it/get_it.dart';
import 'package:lux/lux.dart';
import 'package:lux_flutter/lux_flutter.dart';

GetIt _getIt = GetIt.instance;

class InjectStoreProvider<B extends StateStreamableSource<Object?>> extends StoreProvider<B> {
  InjectStoreProvider({
    Key? key,
    Widget? child,
    bool lazy = true,
  }) : super(
          key: key,
          create: (_) => _getIt<B>(),
          child: child,
          lazy: lazy,
        );
}

class InjectStoreBuilder<B extends StateStreamable<S>, S> extends StoreBuilder<B, S> {
  InjectStoreBuilder({
    Key? key,
    required StoreWidgetBuilder<B, S> builder,
    StoreBuilderCondition<S>? buildWhen,
  }) : super(
          key: key,
          store: _getIt<B>(),
          builder: builder,
          buildWhen: buildWhen,
        );
}

class InjectStoreSelector<B extends StateStreamable<S>, S, T> extends StoreSelector<B, S, T> {
  InjectStoreSelector({
    Key? key,
    required StoreWidgetSelector<S, T> selector,
    required StoreWidgetBuilder<B, T> builder,
  }) : super(
          key: key,
          store: _getIt<B>(),
          selector: selector,
          builder: builder,
        );
}

class InjectStoreWatcher<B extends StateStreamable<S>, S, T> extends StoreWatcher<B, S, T> {
  InjectStoreWatcher({
    Key? key,
    required StoreWidgetWatcher<S, T> watcher,
    required StoreWidgetListStateBuilder<B, T> builder,
  }) : super(
          key: key,
          store: _getIt<B>(),
          watcher: watcher,
          builder: builder,
        );
}

class InjectStoreListener<B extends StateStreamable<S>, S> extends StoreListener<B, S> {
  InjectStoreListener({
    Key? key,
    required StoreWidgetListener<B, S> listener,
    StoreListenerCondition<S>? listenWhen,
    Widget? child,
  }) : super(
          key: key,
          store: _getIt<B>(),
          listener: listener,
          listenWhen: listenWhen,
          child: child,
        );
}

class InjectStoreConsumer<B extends StateStreamable<S>, S> extends StoreConsumer<B, S> {
  InjectStoreConsumer({
    Key? key,
    required StoreWidgetBuilder<B, S> builder,
    required StoreWidgetListener<B, S> listener,
    StoreBuilderCondition<S>? buildWhen,
    StoreListenerCondition<S>? listenWhen,
  }) : super(
          key: key,
          builder: builder,
          listener: listener,
          store: _getIt<B>(),
          buildWhen: buildWhen,
          listenWhen: listenWhen,
        );
}

class InjectTruckProvider<B extends EventStreamableSource<Object?>> extends TruckProvider<B> {
  InjectTruckProvider({
    Key? key,
    Widget? child,
    bool lazy = true,
  }) : super(
          key: key,
          create: (_) => _getIt<B>(),
          child: child,
          lazy: lazy,
        );
}

class InjectTruckListener<B extends EventStreamableSource<S>, S> extends TruckListener<B, S> {
  InjectTruckListener({
    Key? key,
    required TruckWidgetListener<B, S> listener,
    StreamListenerCondition<S>? listenWhen,
    Widget? child,
  }) : super(
          key: key,
          truck: _getIt<B>(),
          listener: listener,
          listenWhen: listenWhen,
        );
}
